@@ -22,7 +22,6 @@ import (
 	"io"
 	"log"
 	"os"
-	"reflect"
 	"testing"
 	"time"
 
@@ -360,8 +359,8 @@ func TestGetBucketPolicy(t *testing.T) {
 			t.Error("Error:", err, bucketName)
 		}
 	}
-	if !reflect.DeepEqual(bucketPolicy, emptyBucketAccessPolicy) {
-		t.Errorf("Bucket policy expected %#v, got %#v", emptyBucketAccessPolicy, bucketPolicy)
+	if bucketPolicy != "" {
+		t.Errorf("Bucket policy expected %#v, got %#v", "", bucketPolicy)
 	}
 
 	err = c.RemoveBucket(bucketName)
@@ -411,7 +410,7 @@ func TestCoreCopyObject(t *testing.T) {
 	objectName := randString(60, rand.NewSource(time.Now().UnixNano()), "")
 	objInfo, err := c.PutObject(bucketName, objectName, bytes.NewReader(buf), int64(len(buf)), "", "", map[string]string{
 		"Content-Type": "binary/octet-stream",
-	})
+	}, nil)
 	if err != nil {
 		t.Fatal("Error:", err, bucketName, objectName)
 	}
@@ -526,7 +525,7 @@ func TestCoreCopyObjectPart(t *testing.T) {
 	objectName := randString(60, rand.NewSource(time.Now().UnixNano()), "")
 	objInfo, err := c.PutObject(bucketName, objectName, bytes.NewReader(buf), int64(len(buf)), "", "", map[string]string{
 		"Content-Type": "binary/octet-stream",
-	})
+	}, nil)
 	if err != nil {
 		t.Fatal("Error:", err, bucketName, objectName)
 	}
@@ -566,7 +565,7 @@ func TestCoreCopyObjectPart(t *testing.T) {
 	}
 
 	// Complete the multipart upload
-	err = c.CompleteMultipartUpload(destBucketName, destObjectName, uploadID, []CompletePart{fstPart, sndPart, lstPart})
+	_, err = c.CompleteMultipartUpload(destBucketName, destObjectName, uploadID, []CompletePart{fstPart, sndPart, lstPart})
 	if err != nil {
 		t.Fatal("Error:", err, destBucketName, destObjectName)
 	}
@@ -673,12 +672,12 @@ func TestCorePutObject(t *testing.T) {
 	metadata := make(map[string]string)
 	metadata["Content-Type"] = objectContentType
 
-	objInfo, err := c.PutObject(bucketName, objectName, bytes.NewReader(buf), int64(len(buf)), "1B2M2Y8AsgTpgAmY7PhCfg==", "", metadata)
+	objInfo, err := c.PutObject(bucketName, objectName, bytes.NewReader(buf), int64(len(buf)), "1B2M2Y8AsgTpgAmY7PhCfg==", "", metadata, nil)
 	if err == nil {
 		t.Fatal("Error expected: error, got: nil(success)")
 	}
 
-	objInfo, err = c.PutObject(bucketName, objectName, bytes.NewReader(buf), int64(len(buf)), "", "", metadata)
+	objInfo, err = c.PutObject(bucketName, objectName, bytes.NewReader(buf), int64(len(buf)), "", "", metadata, nil)
 	if err != nil {
 		t.Fatal("Error:", err, bucketName, objectName)
 	}
@@ -754,7 +753,7 @@ func TestCoreGetObjectMetadata(t *testing.T) {
 	}
 
 	_, err = core.PutObject(bucketName, "my-objectname",
-		bytes.NewReader([]byte("hello")), 5, "", "", metadata)
+		bytes.NewReader([]byte("hello")), 5, "", "", metadata, nil)
 	if err != nil {
 		log.Fatalln(err)
 	}
